@@ -63,11 +63,11 @@ use Search::Typesense::Types qw(
 
 =head1 VERSION
 
-Version 0.02
+VERSION 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 has _ua => (
     is      => 'lazy',
@@ -199,17 +199,6 @@ sub get_collections {
     return $self->_GET( path => [ 'collections', @collection ] );
 }
 
-sub _GET {
-    my ( $self, %arg_for ) = @_;
-    my $request = $arg_for{request};
-
-    my $url = $self->_url( $arg_for{path} );
-
-    my @args = $request ? ( form => $request ) : ();
-    my $tx   = $self->_ua->get( $url, @args );
-    return $self->_check_for_failure( $tx, $arg_for{return_transaction} );
-}
-
 =head2 C<delete_collection>
 
     my $response = $typesense->delete_collection($collection_name);
@@ -225,13 +214,6 @@ sub delete_collection {
     return $self->_DELETE( path => [ 'collections', $collection ] );
 }
 
-sub _DELETE {
-    my ( $self, %arg_for ) = @_;
-    my $url = $self->_url( $arg_for{path} );
-    my $tx  = $self->_ua->delete($url);
-    return $self->_check_for_failure( $tx, $arg_for{return_transaction} );
-}
-
 =head2 C<create_collection>
 
     my $collection = $typesense->create_collection(\%definition);
@@ -260,20 +242,6 @@ sub create_collection {
     );
 } ## end sub create_collection
 
-sub _POST {
-    my ( $self, %arg_for ) = @_;
-    my $request = $arg_for{request};
-    my $url     = $self->_url( $arg_for{path} )->query( $arg_for{query} || {} );
-
-    my @args = ref $request
-      ? ( json => $request )    # data structures
-      : $request ? $request     # raw POST content
-      :            croak("POST $url not allowed without a request");
-
-    my $tx = $self->_ua->post( $url, @args );
-    return $self->_check_for_failure( $tx, $arg_for{return_transaction} );
-}
-
 =head2 C<create_document>
 
     my $document = $typesense->create_document($collection, \%data);
@@ -333,21 +301,6 @@ sub update_document {
     );
 }
 
-# Yet, we're getting some serious duplication here. Need to refactorit.
-sub _PATCH {
-    my ( $self, %arg_for ) = @_;
-    my $request = $arg_for{request};
-    my $url     = $self->_url( $arg_for{path} )->query( $arg_for{query} || {} );
-
-    my @args = ref $request
-      ? ( json => $request )    # data structures
-      : $request ? $request     # raw POST content
-      :            croak("PATCH $url not allowed without a request");
-
-    my $tx = $self->_ua->patch( $url, @args );
-    return $self->_check_for_failure( $tx, $arg_for{return_transaction} );
-}
-
 =head2 C<delete_document>
 
     my $document = $typesense->delete_document($collection_name, $document_id);
@@ -476,11 +429,56 @@ sub delete_all_collections {
     }
 }
 
-sub _check_for_failure {
-    my ( $self, $tx, $return_transaction ) = @_;
-    my $res = $tx->res;
-    unless ( $res->is_success ) {
+sub _GET {
+    my ( $self, %arg_for ) = @_;
+    my $request = $arg_for{request};
+    my @args    = $request ? ( form => $request ) : ();
+    return $self->_handle_request( \%arg_for, \@args );
+}
 
+sub _DELETE {
+    my ( $self, %arg_for ) = @_;
+    return $self->_handle_request( \%arg_for );
+}
+
+sub _POST {
+    my ( $self, %arg_for ) = @_;
+    my $request = $arg_for{request};
+    my @args    = ref $request ? ( json => $request ) : $request;
+    return $self->_handle_request( \%arg_for, \@args );
+}
+
+sub _PATCH {
+    my ( $self, %arg_for ) = @_;
+    my $request = $arg_for{request};
+    my @args    = ref $request ? ( json => $request ) : $request;
+    return $self->_handle_request( \%arg_for, \@args );
+}
+
+sub _handle_request {
+    my ( $self, $arg_for, $args ) = @_;
+
+    # We must only be called by methods like _GET, _POST, _DELETE, and so on.
+    # We strip the package name and leading underscore
+    # (Search::Typesense::_GET becomes GET) and then we call lc() on what's
+    # left. That becomes our HTTP verb and the $check verifies that this is an
+    # allowed verb.
+    my ( undef, undef, undef, $method ) = caller(1);
+    $method =~ s/^.*::_//;
+    state $check = compile( Enum [qw/get delete post patch/] );
+    ($method) = $check->( lc $method );
+
+    # make the actual request, passing a query string, if any, and passing any
+    # args, if any (those can become part of a query string for GET, or part
+    # of the body for other HTTP verbs
+    my @args = $args ? @$args : ();
+    my $url = $self->_url( $arg_for->{path} )->query( $arg_for->{query} || {} );
+    my $tx  = $self->_ua->$method( $url, @args );
+    my $res = $tx->res;
+
+    # If the response is not succesful, return nothing if it's a 404.
+    # Otherwise, croak()
+    unless ( $res->is_success ) {
         return if ( $res->code // 0 ) == 404;
         my $message = $res->message // '';
 
@@ -489,7 +487,8 @@ sub _check_for_failure {
         my $url    = $tx->req->url;
         croak("'$method $url' failed: $message. $body");
     }
-    return $return_transaction ? $tx : $tx->res->json;
+
+    return $arg_for->{return_transaction} ? $tx : $tx->res->json;
 } ## end sub _check_for_failure
 
 =head1 AUTHOR
@@ -498,9 +497,10 @@ Curtis "Ovid" Poe, C<< <ovid at allaroundtheworld.fr> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-search-typesense at rt.cpan.org>, or through
-the web interface at L<https://rt.cpan.org/NoAuth/ReportBug.html?Queue=Search-Typesense>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+Please report any bugs or feature requests to
+C<https://github.com/Ovid/Search-Typesense/issues>.  I will be notified, and
+then you'll automatically be notified of progress on your bug as I make
+changes.
 
 =head1 SUPPORT
 
@@ -508,7 +508,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc Search::Typesense
 
-
 You can also look for information at:
 
 =over 4
